@@ -32,6 +32,7 @@ type Cluster struct {
 	IngressVirtualRouterID uint8
 	VIPNetmask             int
 	MasterAmount           int64
+	EtcdBackends           string
 }
 
 type Backend struct {
@@ -41,10 +42,11 @@ type Backend struct {
 }
 
 type ApiLBConfig struct {
-	ApiPort  uint16
-	LbPort   uint16
-	StatPort uint16
-	Backends []Backend
+	ApiPort      uint16
+	LbPort       uint16
+	StatPort     uint16
+	Backends     []Backend
+	FrontendAddr string
 }
 
 type Node struct {
@@ -57,8 +59,8 @@ type Node struct {
 	DNSUpstreams      []string
 }
 
-func getDNSUpstreams() (upstreams []string, err error) {
-	dnsFile, err := os.Open("/etc/resolv.conf")
+func getDNSUpstreams(resolvConfPath string) (upstreams []string, err error) {
+	dnsFile, err := os.Open(resolvConfPath)
 	if err != nil {
 		return upstreams, err
 	}
@@ -142,7 +144,7 @@ func getClusterConfigMapInstallConfig(configPath string) (installConfig types.In
 	return ic, err
 }
 
-func GetConfig(kubeconfigPath, clusterConfigPath string, apiVip net.IP, ingressVip net.IP, dnsVip net.IP, apiPort, lbPort, statPort uint16) (node Node, err error) {
+func GetConfig(kubeconfigPath, clusterConfigPath, resolvConfPath string, apiVip net.IP, ingressVip net.IP, dnsVip net.IP, apiPort, lbPort, statPort uint16) (node Node, err error) {
 	// Try cluster-config.yml first
 	clusterName, clusterDomain, err := getClusterConfigClusterNameAndDomain(clusterConfigPath)
 	if err != nil {
@@ -208,14 +210,14 @@ func GetConfig(kubeconfigPath, clusterConfigPath string, apiVip net.IP, ingressV
 	}
 	node.NonVirtualIP = nonVipAddr.IP.String()
 
-	resolvConfUpstreams, err := getDNSUpstreams()
+	resolvConfUpstreams, err := getDNSUpstreams(resolvConfPath)
 	if err != nil {
 		return node, err
 	}
 	// Filter out our potential CoreDNS addresses from upstream servers
 	node.DNSUpstreams = make([]string, 0)
 	for _, upstream := range resolvConfUpstreams {
-		if upstream != node.NonVirtualIP && upstream != node.Cluster.DNSVIP && upstream != "127.0.0.1" {
+		if upstream != node.NonVirtualIP && upstream != node.Cluster.DNSVIP && upstream != "127.0.0.1" && upstream != "::1" {
 			node.DNSUpstreams = append(node.DNSUpstreams, upstream)
 		}
 	}
@@ -225,10 +227,20 @@ func GetConfig(kubeconfigPath, clusterConfigPath string, apiVip net.IP, ingressV
 	node.VRRPInterface = vipIface.Name
 
 	domain := fmt.Sprintf("%s.%s", clusterName, clusterDomain)
-	node.LBConfig, err = GetLBConfig(domain, apiPort, lbPort, statPort)
+	node.LBConfig, err = GetLBConfig(domain, apiPort, lbPort, statPort, apiVip)
 	if err != nil {
 		return node, err
 	}
+	etcdBackends, err := getSortedBackends(domain)
+	if err != nil {
+		return node, err
+	}
+	for _, backend := range etcdBackends {
+		if len(node.Cluster.EtcdBackends) > 0 {
+			node.Cluster.EtcdBackends += ","
+		}
+		node.Cluster.EtcdBackends += fmt.Sprintf("https://%s:2379", strings.TrimRight(backend.Host, "."))
+	}
 
 	return node, err
 }
@@ -259,12 +271,17 @@ func getSortedBackends(domain string) (backends []Backend, err error) {
 	return backends, err
 }
 
-func GetLBConfig(domain string, apiPort, lbPort, statPort uint16) (ApiLBConfig, error) {
+func GetLBConfig(domain string, apiPort, lbPort, statPort uint16, apiVip net.IP) (ApiLBConfig, error) {
 	config := ApiLBConfig{
 		ApiPort:  apiPort,
 		LbPort:   lbPort,
 		StatPort: statPort,
 	}
+	// LB frontend address: IPv6 '::' , IPv4 ''
+	if apiVip.To4() == nil {
+		config.FrontendAddr = "::"
+	}
+
 	backends, err := getSortedBackends(domain)
 	if err != nil {
 		log.WithFields(logrus.Fields{
